///////////////////////////////////////////////////////////////////////////////
//  Filename: $RCSfile: HungarianInterface.java,v $
//  Purpose:  Atom representation.
//  Language: Java
//  Compiler: JDK 1.4
//  Authors:  Joerg K. Wegner
//  Version:  $Revision: 1.3 $
//            $Date: 2004/09/07 08:28:05 $
//            $Author: wegnerj $
//
//  Copyright (c) Dept. Computer Architecture, University of Tuebingen, Germany
///////////////////////////////////////////////////////////////////////////////
package Alignment;

import java.io.File;


/**
 * This interface class to Hungarian defines some native methods for
 * accessing parts of the Hungarian functionality.
 *
 * @author     wegnerj
 */
public class HungarianInterface
{
    //~ Static fields/initializers /////////////////////////////////////////////

    /** Obtain a suitable logger. */
    private static HungarianInterface hungarian = null;
    private static String hungarianLibraryPath = "lib";
    private static String hungarianLibraryName = "hungarian";
    private static boolean addOSNameFlag = true;

    //~ Constructors ///////////////////////////////////////////////////////////

    /** Don't let anyone instantiate this class */
    private HungarianInterface()
    {
    }

    //~ Methods ////////////////////////////////////////////////////////////////

    /**
     * Sets the name of the OElib interface dynamic library. Note that the name can
     * not be changed anymore after the first call to the 'instance()' method.
     * The default library name is 'OE_jni', which should in almost all cases be ok.
     */
    public static void setLibraryName(String libraryName)
    {
        // set new library name if the OElib interface class instance was not already created
        if (hungarian == null)
        {
            hungarianLibraryName = libraryName;
        }
    }

    /**
     * Sets the path to the OElib interface dynamic library. Note that the path can
     * not be changed anymore after the first call to the 'instance()' method.
     *  The default library path is 'lib'.
     */
    public static void setLibraryPath(String libraryPath)
    {
        // set new library path if the OElib interface class instance was not already created
        if (hungarian == null)
        {
            hungarianLibraryPath = libraryPath;
        }
    }

    /** @todo maybe move this method to a more common class */
    public static String getOperationSystemName()
    {
        String osName = System.getProperty("os.name");

        // determine name of operation system and convert it into lower caps without blanks
        if (osName.indexOf("Windows") != -1)
        {
            osName = "windows";
        }
        else if (osName.indexOf("Linux") != -1)
        {
            osName = "linux";
        }

        return osName;
    }

    /**
     * Determines if the OS name (like 'windows' or 'linux') will be appended
     * automatically to the library path. Default value is 'true'.
     */
    public static void addOSNameToLibraryPath(boolean value)
    {
        addOSNameFlag = value;
    }

    /**
     * Tries to create an instance of our interface class.
     */
    public static synchronized HungarianInterface instance()
    {
        // check if we already have an instance of the OElib interface class
        if (hungarian != null)
        {
            return hungarian;
        }

        String libPath = null;

        try
        {
            // build the full path to the OElib interface dynamic library
            libPath = hungarianLibraryPath +
                System.getProperty("file.separator", "/");

            if (addOSNameFlag)
            {
                libPath += (getOperationSystemName() +
                System.getProperty("file.separator", "/"));
            }

            libPath += System.mapLibraryName(hungarianLibraryName);

            // make sure that we have the absolute path
            libPath = new File(libPath).getAbsolutePath();
        }
         catch (Exception e)
        {
         	System.out.println("Error!");
        }

        //System.out.println("Try loading "+libPath);
        try
        {
            // try to load the Hungarian java natice interface dynamic library
            //System.loadLibrary(hungarianLibraryName);
            //System.loadLibrary(libPath);
            //System.out.println("Before load");
            System.load(libPath);

            //System.out.println("After load");
        }
         catch (Error e)
        {
         	System.out.println("Error!");
        }

        //System.out.println("Try getting instance "+libPath);
        try
        {
            // try to create instance of the oelib interface
            hungarian = new HungarianInterface();
        }
         catch (Exception e)
        {
         	System.out.println("Error!");
        }

        return hungarian;
    }

    public native double solveHungarianAssigment(double[][] costMatrix,
        boolean maximize);
}
///////////////////////////////////////////////////////////////////////////////
//  END OF FILE.
///////////////////////////////////////////////////////////////////////////////
