/*
 * Created on 31.01.2008
 */
package Parser;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Vector;

import Utilities.Coords;
import Utilities.Graph;

/**
 * @author Thomas Fober
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         thomas@mathemetik.uni-marburg.de
 */
public class PTCParser{
	public static Graph[] parsePTCGraphs(String path, double delta, boolean bond){
		String s;
		String[] nums;
		Vector<Graph> graphsVector = new Vector<Graph>();
		Vector<Coords> coords = new Vector<Coords>();
		Vector<Integer> hAtome = new Vector<Integer>();
		Vector<Integer> map = new Vector<Integer>();
		int number = -1;
		try{
			BufferedReader file = new BufferedReader(new FileReader(path));
			s = file.readLine();
			while(s != null){
				if(s.length() >= 17 && s.substring(0, 17).compareTo("@<TRIPOS>MOLECULE") == 0){
					number++;
					//skip useless information - collect useful information
					s = file.readLine();
					s = file.readLine();
					nums = s.trim().split("\\s+");
					int numberNodes = Integer.parseInt(nums[0]);
					int numberBonds = Integer.parseInt(nums[1]);
					s = file.readLine();
					s = file.readLine();
					s = file.readLine();
					s = file.readLine();
					s = file.readLine();
					//from here on: AtomData
					for (int i = 0; i < numberNodes; i++){
						s = file.readLine().trim();
						String[] lineC = s.split("\\s+");
						Coords actual = new Coords(Double.parseDouble(lineC[2]), Double.parseDouble(lineC[3]), Double.parseDouble(lineC[4]), lineC[5]);
						map.addElement(Integer.parseInt(lineC[0]));
						coords.addElement(actual);
					}			
//					Creating distance matrix
					double[][] distance = new double[coords.size()][coords.size()];
					for (int i = 0; i < coords.size();i++){
						for (int j = 0; j < coords.size();j++){
							if (i==j) distance[i][j] = 0;
							else distance[i][j] = Math.sqrt(Math.pow((coords.get(i).getX()-coords.get(j).getX()),2) + Math.pow((coords.get(i).getY()-coords.get(j).getY()),2) + Math.pow((coords.get(i).getZ()-coords.get(j).getZ()),2));
						}
					}
					//creating bond matrix
					file.readLine();
					double[][] bonds = new double[map.size()][map.size()];
					for (int i=0; i < numberBonds; i++){
						String next = file.readLine().trim();
						String[] lineC = next.split("\\s+");
						int xa = Integer.parseInt(lineC[1]);
						int ya = Integer.parseInt(lineC[2]);
						if (hAtome.contains(xa) || hAtome.contains(ya)) continue;
						else {
							int x = map.indexOf(xa);
							int y = map.indexOf(ya);
							if (lineC[3].equals("ar"))bonds[x][y] = 4;
							else if (lineC[3].equals("am"))bonds[x][y] = 5;
							else bonds[x][y] = Integer.parseInt(lineC[3]);
						}
					}
					for (int i = 0; i < distance.length; i++){
						for (int j = 0; j < distance[0].length; j++){
							if (distance[i][j] > delta) distance[i][j] = Double.POSITIVE_INFINITY;
						}
					}
					Graph g = new Graph();
					//g.bonds = bonds;
					g.coords = coords;
					g.distances = distance;
					if(bond) g.distances = bonds;
					g.numberOfNodes = coords.size();
					g.path = "TEST";
					g.name = ((Integer)number).toString();
					g.nodes = new String[coords.size()];
					for (int i = 0; i < coords.size(); i++){
						g.nodes[i] = coords.get(i).getLabel();
					}
					graphsVector.add(g);
					nums = null;
					coords.clear();
					hAtome.clear();
					map.clear();
				}
//				next entry
				s = file.readLine();
			}
		}catch(Exception e){
			System.out.println("Error reading graphs");
		}
		Graph[] graphs = new Graph[graphsVector.size()];
		for(int i = 0; i < graphsVector.size(); i++){
			graphs[i] = graphsVector.get(i);
		}
		return graphs;
	}
	
	public static int[] parsePTCResults(String path){
		Vector<Integer> resTmp = new Vector<Integer>();
		String s;
		try{
			BufferedReader file = new BufferedReader(new FileReader(path));
			s = file.readLine();
			while(s != null){
				resTmp.add(Integer.parseInt(s));
				s = file.readLine();
			}
		}catch(Exception e){
			System.out.println("Error reading target");
		}
			
		int[] result = new int[resTmp.size()];
		for(int i = 0; i < resTmp.size(); i++){
			result[i] = resTmp.get(i);
		}
		return result;
	}
}
