/*
 * Created on 10.08.2007
 *
 */
package Parser;
import java.io.*;
import Utilities.Graph;
/**
 * @author Thomas Fober, Marco Mernberger
 * 		   Fachbereich Mathematik und Informatik
 * 		   Philipps-Universitt Marburg
 * 		   {thomas, mernberger}@mathematik.uni-marburg.de
 */
public class ParseGraphs {
	
	
	/**Generates a random permutation
	 * 
	 * @param length number of elements
	 * @return permutated integer array 
	 */
	private static Integer[] randPerm(int length){
		Integer[] randPerm = new Integer[length];
		for(int i = 0; i < length; i++){
			randPerm[i] = i;
		}
		for (int i = 0; i < length; i++) {
	         int r = (int)(Math.random() * (i+1));
	         int swap = randPerm[r];
	         randPerm[r] = randPerm[i];
	         randPerm[i] = swap;
		}
		return randPerm;
	}
	
	/**Randomly creates an array of graphs from a given input directory
	 * 
	 * @param dirname path to input directory
	 * @param delta cutoff value for vertex distances
	 * @param sum number of graphs
	 * @return Array of graphs
	 * @throws IOException if invalid input directory
	 */
	public static Graph[] createGraphs(String dirname, int delta, int sum) throws IOException{
		File dir = new File(dirname);
		String[] contents = dir.list();
		if (contents.length < sum) {
			System.out.println("Warning: Input directory doesn't contain enough files, adjusting sum of graphs to maximum ...");
			sum = contents.length;
		}
		if (contents==null) throw new IOException("Directory empty or invalid path");
		Integer[] random = randPerm(contents.length); 
		
		Graph[] ret = new Graph[sum];
		for (int i = 0; i < sum; i++){
			FileParser c = null;
			if (contents[random[i]].endsWith(".rlbcoor")) c = new RlbCoorParser();
			else if (contents[random[i]].endsWith(".mol2")) c = new ChemParser();
			else if ((contents[random[i]].endsWith(".pseudoc.txt"))) c = new CavityParser();
			else if ((contents[random[i]].endsWith(".graph"))) c = new CommonParser();
			else continue;
			Graph graph = c.createGraph(dirname+"\\"+contents[random[i]], delta);
			ret[i] = graph;
		}
		return ret;
	}
	
	/**Generates a graph from a given input file
	 * 
	 * @param pathtofile path
	 * @param delta cutoff value
	 * @return graph object
	 */
	public static Graph parseFile(String pathtofile, double delta) {
		FileParser c = null;
		if (pathtofile.endsWith(".rlbcoor")) c = new RlbCoorParser();
		else if (pathtofile.endsWith(".mol2")) c = new ChemParser();
		else if (pathtofile.endsWith(".pseudoc.txt")) c = new CavityParser();
		else if (pathtofile.endsWith(".graph")) c = new CommonParser();
		else return null;
		return c.createGraph(pathtofile, delta);
	}
	
	/**Tests, wether a file is parsable
	 * 
	 * @param file file path
	 * @return true, if file is parsable
	 */
	public static boolean isParsable(String file) {
		return file.endsWith(".mol2") || file.endsWith(".pseudoc.txt") || file.endsWith(".rlbcoor") || file.endsWith(".graph");
	}
}
