/*
 * Created on 23.10.2007
 */
package RealValue;
import java.util.ArrayList;
import java.util.Random;
import java.util.Vector;

import Utilities.*;


/**
 * @author Thomas Fober
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         thomas@mathemetik.uni-marburg.de
 */
public class ES {
	
	private static Individual mutation(Individual ind, double cTau, boolean nStepSizes, Vector<Coords>[] molecules, int membership, double sigma){
		Random rand = new Random();
		Individual child = new Individual(nStepSizes, molecules, ind.name, ind.group, ind.typ);
		if(nStepSizes){
			double tau = 1 / Math.sqrt(2 * Math.sqrt(2 * 6 * (ind.molecules.length - 1)));
			double tau0 = 1 / Math.sqrt(2 * 6 * (ind.molecules.length - 1));
			double tmp = Math.exp(tau0 * rand.nextGaussian());
			for(int i = 0; i < ind.stepsizes.length; i++){
				child.stepsizes[i] = tmp * ind.stepsizes[i] * Math.exp(tau * rand.nextGaussian());
				child.object[i] = ind.object[i] + child.stepsizes[i] * rand.nextGaussian();
			}
		}else{
			double tau = 1 / Math.sqrt(Math.sqrt(2 * 6 * (ind.molecules.length - 1)));
			child.stepsizes[0] = ind.stepsizes[0] * Math.exp(tau * rand.nextGaussian());
			for(int i = 0; i < ind.stepsizes.length; i++){
				child.object[i] = ind.object[i] + child.stepsizes[0] * rand.nextGaussian();
			}
		}
		child.evaluate(membership, sigma);
		//Plateaus
		if(ind.fitness == child.fitness){
			double s = 0.5;
			while(child.fitness == ind.fitness){
				s = s * 2;
				for(int i = 0; i < 6; i++){
					child.object[i] = child.object[i] + s; 
				}
				child.evaluate(membership, sigma);
			}
			Individual tmp = child;
			while(child.fitness == ind.fitness){
				s = s * 2;
				for(int i = 0; i < 6; i++){
					child.object[i] = child.object[i] - s; 
				}
				child.evaluate(membership, sigma);
			}
			if(tmp.fitness > child.fitness) child = tmp;
			
		}
		return child;
	}
	private static boolean valid(Individual ind){
		for(int i = 0; i < ind.molecules.length - 1; i++){
			if(ind.object[i*6 + 0] > Double.POSITIVE_INFINITY || ind.object[i*6 + 0] < Double.NEGATIVE_INFINITY){
				return false;
			}
			if(ind.object[i*6 + 1] > Double.POSITIVE_INFINITY || ind.object[i*6 + 1] < Double.NEGATIVE_INFINITY){
				return false;
			}
			if(ind.object[i*6 + 2] > Double.POSITIVE_INFINITY || ind.object[i*6 + 2] < Double.NEGATIVE_INFINITY){
				return false;
			}
			if(ind.object[i*6 + 3] > 2 * Math.PI || ind.object[i*6 + 3] < 0){
				return false;
			}
			if(ind.object[i*6 + 4] > 2 * Math.PI || ind.object[i*6 + 4] < 0){
				return false;
			}
			if(ind.object[i*6 + 5] > 2 * Math.PI || ind.object[i*6 + 5] < 0){
				return false;
			}
		}
		return true;
	}
	private static Individual recombination(Individual[] subPop, int rho, boolean intermediateX, boolean intermediateS){
		Random rand = new Random();
		Individual child = new Individual(subPop[0].nStepSizes, subPop[0].molecules, subPop[0].name, subPop[0].group, subPop[0].typ);
		if(rho == 1) return subPop[0];
		if(intermediateX){
			for(int i = 0; i < subPop[0].object.length; i++){
				for(int j = 0; j < rho; j++){
					child.object[i] = child.object[i] + subPop[j].object[i] / rho; 
				}
			}
		}
		if(intermediateS){
			for(int i = 0; i < subPop[0].stepsizes.length; i++){
				child.stepsizes[i] = 0;
				for(int j = 0; j < rho; j++){
					child.stepsizes[i] = child.stepsizes[i] + subPop[j].stepsizes[i] / rho; 
				}
			}
		}
		if(!intermediateX){
			int index;
			for(int i = 0; i < subPop[0].object.length; i++){
				index = rand.nextInt(rho);
				child.object[i] = subPop[index].object[i]; 
			}
		}
		if(!intermediateS){
			int index;
			for(int i = 0; i < subPop[0].stepsizes.length; i++){
				index = rand.nextInt(rho);
				child.stepsizes[i] = subPop[index].stepsizes[i]; 
			}
		}
		return child;
	}
	private static Individual[] matingSelection(int rho, Individual[] pop){
		Random rand = new Random();
		Individual[] subPop = new Individual[rho];
		int index;
		for(int i = 0; i < rho; i++){
			index = rand.nextInt(pop.length);
			subPop[i] = pop[index].clone();
		}
		return subPop;
	}
	private static Individual[] createChildren(double nu, Individual[] pop, int rho, double cTau, boolean intermediateX, boolean intermediateS, boolean nStepSizes, int membership, double sig){
		int lambda = (int)Math.round(pop.length * nu);
		Individual[] children = new Individual[lambda];
		for(int i = 0; i < lambda; i++){
			Individual[] tmp1 = new Individual[rho];
			tmp1 = matingSelection(rho, pop);
			Individual tmp2 = recombination(tmp1, rho, intermediateX, intermediateS);
			children[i] = mutation(tmp2, cTau, nStepSizes, pop[0].molecules, membership, sig);
			while(!valid(children[i])){
				tmp1 = matingSelection(rho, pop);
				tmp2 = recombination(tmp1, rho, intermediateX, intermediateS);
				children[i] = mutation(tmp2, cTau, nStepSizes, pop[0].molecules, membership, sig);
			}
			//children[i].evaluate(membership, sig);
		}
		return children;
	}
	private static Individual[] selection(int mu, Individual[] children, Individual[] parents, int kappa){
		ArrayList<Individual> pool = new ArrayList<Individual>();
		for(int i = 0; i < parents.length; i++){
			if(parents[i].age < kappa){
				pool.add(parents[i]);
				parents[i].age++;
			}
		}
		for(int i = 0; i < children.length; i++){
			pool.add(children[i]);
		}
		Individual[] nextGeneration = new Individual[mu];
		double bestFitness;
		Individual bestInd = null;
		for(int i = 0; i < mu; i++){
			bestFitness = Double.NEGATIVE_INFINITY;
			for(Individual ind: pool){
				if(ind.fitness > bestFitness){
					bestFitness = ind.fitness;
					bestInd = ind;
				}
			}
			nextGeneration[i] = bestInd.clone();
			bestInd.fitness = Double.NEGATIVE_INFINITY;
		}
		return nextGeneration;
	}
	private static Individual[] initialize(int mu, double[][] sigma0, double[][] domain, Vector<Coords>[] molecules, boolean nStepSizes, String name, Individual solution, int membership, double sig){
		Random rand = new Random();
		Individual[] startPop = new Individual[mu];
		if(domain[0].length == 1){
			double[][] tmp = new double[2][6 * (molecules.length - 1)];
			for(int i = 0; i < 6 * (molecules.length - 1); i++){
				tmp[0][i] = domain[0][0];
				tmp[1][i] = domain[1][0];
			}
			domain = tmp;
		}
		if(sigma0[0].length == 1 && nStepSizes){
			double[][] tmp = new double[2][6 * (molecules.length - 1)];
			for(int i = 0; i < 6 * (molecules.length - 1); i++){
				tmp[0][i] = sigma0[0][0];
				tmp[1][i] = sigma0[1][0];
			}
			sigma0 = tmp;
		}
		int number;
		if(solution == null){
			number = 0;
		}else{
			number = 1;
			startPop[0] = solution;
		}
		for(int i = number; i < mu; i++){
			startPop[i] = i == 0 ? new Individual(nStepSizes, molecules, name, null, null) : new Individual(nStepSizes, molecules, name, startPop[0].group, startPop[0].typ);
			for(int j = 0; j < startPop[i].object.length; j++){
				startPop[i].object[j] = domain[0][j] + rand.nextDouble() * (domain[1][j] - domain[0][j]);
				if(j % 6 == 3 || j % 6 == 4 || j % 6 == 5){
					//startPop[i].object[j] = 0;
					startPop[i].object[j] = 0 + rand.nextDouble() * 2 * Math.PI;
				}
			}
			for(int j = 0; j < startPop[i].stepsizes.length; j++){
				startPop[i].stepsizes[j] = sigma0[0][j] + rand.nextDouble() * (sigma0[1][j] - sigma0[0][j]);
			}
			startPop[i].evaluate(membership, sig);
		}
		return startPop;
	}
	private static Individual[] loop(int mu, double nu, int kappa, int rho, Individual[] parents, double cTau, boolean intermediateX, boolean intermediateS, boolean nStepSizes, int membership, double sig){
		return selection(mu, createChildren(nu, parents, rho, cTau, intermediateX, intermediateS, nStepSizes, membership, sig), parents, kappa);
	}
	
	public static Individual solver(int mu, double nu, int kappa, int rho, double[][] sigma0, double[][] object, double cTau, boolean intermediateX, boolean intermediateS, boolean nStepSizes, double time, int gen, int stallGen, double stallTime, double fitness, double stepsize, Vector<Coords>[] molecules, String name, Individual solution, int membership, double sig){
		Individual[] pop = initialize(mu, sigma0, object, molecules, nStepSizes, name, solution, membership, sig);
		Individual best = pop[0];
		Individual[] children;
		long startTime = System.currentTimeMillis();
		long currentTime = System.currentTimeMillis();
		boolean terminate = false;
		int generation = 0;
		int genLastImpr = 0;
		long timeLastImpr = 0;
		while(!terminate){
			//System.out.println("Generation: " + generation);
			pop = loop(mu, nu, kappa, rho, pop, cTau, intermediateX, intermediateS, nStepSizes, membership, sig);
			currentTime = System.currentTimeMillis();
			generation++;
			for(Individual i : pop){
				if(best.fitness < i.fitness){
					best = i.clone();
					genLastImpr = generation;
					timeLastImpr = System.currentTimeMillis();
				}
			}
			if(generation > gen) terminate = true;
			if(best.fitness >= fitness){terminate = true; System.out.println("Fitness");}
			if(-timeLastImpr + currentTime > stallTime){terminate = true; System.out.println("Stall Time");}
			if(generation - genLastImpr > stallGen){terminate = true; System.out.println("Stall Generation");}
			if(maxStepSize(pop) < stepsize) terminate = true;
		}
		best.time = currentTime - startTime;
		return best;
	}
	
	private static double maxStepSize(Individual[] pop){
		double max = 0;
		for(int i = 0; i < pop.length; i++){
			for(int j = 0; j < pop[0].stepsizes.length; j++){
				if(pop[i].stepsizes[j] > max) max = pop[i].stepsizes[j];
			}
		}
		return max;
	}	
}
