/**18.12.2007
 * 12:20:28
 * 2007
 * Description:
 */
package Utilities;

import java.util.Collection;
import java.util.Set;
import java.util.Vector;
import java.util.HashMap;
import java.util.Iterator;

/**
 * @author Marco Mernberger, 
 * 		   Philipps-Universitt Marburg, 
 * 		   Fachbereich Mathematik und Informatik ,
 * 		   mernberger@mathematik.uni-marburg.de
 */
public class Bag<E> implements Collection<E> {

	private HashMap<E, Integer> bag;
	private int size;
	
	
	public Bag() {
		this.bag = new HashMap<E, Integer>();
	}
	
	
	public Bag(Collection<E> col) {
		super();
		this.size = col.size();
		for (E item : col) {
			if (this.bag.containsKey(item))
				this.bag.put(item, this.bag.get(item) + 1);
			else
				this.bag.put(item, 1);
		}
	}

	
	public Set<E> keySet(){
		return this.bag.keySet();
	}
	
	
	public E getMax() {
		E max = null;
		int count = 0;
		for (E key : this.bag.keySet()) {
			if (this.bag.get(key) > count) {
				count = this.bag.get(key);
				max = key;
			}
		}
		return max;
	}
	
	public int getCount(E key) {
		return this.bag.get(key);
	}
	
	public int getMaxCount() {
		return this.bag.get(this.getMax());
	}
	
	public boolean add(E o) {
		if (this.bag.containsKey(o)) this.bag.put(o, this.bag.get(o) + 1);
		else this.bag.put(o, 1);
		this.size++;
		return true;
	}

	public boolean addAll(Collection<? extends E> c) {
		for (E item : c) {
			if (this.bag.containsKey(item)) this.bag.put(item, this.bag.get(item) + 1);
			else this.bag.put(item, 1);
			this.size++;
		}
		return true;
	}

	public void clear() {
		this.bag.clear();
		this.size = 0;
	}

	public boolean contains(Object o) {
		return this.bag.containsKey(o);
	}

	public boolean containsAll(Collection<?> c) {
		return this.bag.keySet().containsAll(c);
	}

	public boolean isEmpty() {
		return this.bag.isEmpty();
	}

	public Iterator<E> iterator() {
		Vector<E> list = new Vector<E>();
		for(E key : this.bag.keySet()) {
			int sum = this.bag.get(key);
			for (int i = 0; i < sum; i++) {
				list.add(key);
			}
		}
		return this.bag.keySet().iterator();
	}

	public boolean remove(Object o) {
		if (!this.bag.containsKey(o)) return false;
		else if (this.bag.get(o)==1) this.bag.remove(o);
		else this.bag.put((E)o, this.bag.get(o)-1);
		this.size--;
		return true;
	}

	public boolean removeAll(Collection<?> c) {
		for(Object o : c) {
			this.remove(o);
			this.size--;
		}
		return true;
	}

	public boolean retainAll(Collection<?> c) {
		for(E key : this.bag.keySet()) this.bag.put(key, 0);
		
		for(E key : this.bag.keySet()) {
			if (!c.contains(key)) this.bag.remove(key);
			else {
				this.bag.put(key, this.bag.get(key)+1);
				c.remove(key);
			}
		}
		return true;
	}

	public int size() {
		int size = 0;
		for(E key : this) size += this.getCount(key);
		return size;
	}

	public Object[] toArray() {
		Object[] ret = new Object[this.size];
		int x = 0;
		for(E key : this.bag.keySet()) {
			for (int i = 0; i < this.bag.get(key); i++){
				ret[x] = key;
				x++;
			}
		}
		return ret;
	}

	public <T> T[] toArray(T[] a) {
		int x = 0;
		for(E key : this.bag.keySet()) {
			for (int i = 0; i < this.bag.get(key); i++){
				a[x] = (T)key;
				x++;
			}
		}
		return a;
	}

}
