/**14.01.2008
 * 12:22:36
 * 2008
 * Description:
 */
package Utilities;

/**
 * @author Thomas Fober & Marco Mernberger
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         {thomas, mernberger}@mathemetik.uni-marburg.de
 *
 */
public class ClassicEvaluation {

	/**Evaluates a given MGA Solution1
	 * 
	 * @param solution MGA Solution1
	 * @param parameter set of scoring parameter
	 * @return score of MGA
	 */
	public static double evaluate(Solution solution, EvalParameter parameter) {
		return evaluate(solution.solution, solution.graphs, parameter);
	}
	
	/**Evaluates a given MGA Solution1 by using default parameter
	 * 
	 * @param solution MGA Solution1
	 * @return score of MGA
	 */
	public static double evaluate(Solution solution) {
		return evaluate(solution, new EvalParameter());
	}
	
	/**Evaluates a given MGA Individual
	 * 
	 * @param ind MGA individual
	 * @return score of MGA
	 */
//	public static double evaluate(Individual ind) {
//		return evaluate(ind.alignment, ind.graphs, ind.parameter);
//	}
	
	/**Evaluates a given MGA Individual
	 * 
	 * @param alignment int[][] storing the alignment
	 * @param parameter parameter for evaluation (costs for editing)
	 * @param graphs graphs aligned in alignment
	 * @return score of MGA
	 */
	public static double evaluate(int[][] alignment, Graph[] graphs, EvalParameter parameter){
		return nodeScore(alignment, graphs, parameter) + edgeScore(alignment, graphs, parameter);
	}	
	
	/**Calculates edge score.
	 * 
	 */
	private static double edgeScore(int[][] alignment, Graph[] graphs, EvalParameter parameter){
		int columns = alignment[0].length;
		int rows = alignment.length;
		double edgeScore = 0;
		for(int i = 0; i < columns; i++){
			for(int j = i+1; j < columns; j++){
				for(int k = 0; k < rows; k++){
					for(int l = k+1; l < rows; l++){
						if(alignment[k][i] != -1 && alignment[k][j] != -1 && alignment[l][i] != -1 && alignment[l][j] != -1){
							if(Math.abs(graphs[k].distances[alignment[k][i]][alignment[k][j]] - graphs[l].distances[alignment[l][i]][alignment[l][j]]) <= parameter.epsilon){
								edgeScore = edgeScore + parameter.edgeMatch;
							}
							if(Math.abs(graphs[k].distances[alignment[k][i]][alignment[k][j]] - graphs[l].distances[alignment[l][i]][alignment[l][j]]) > parameter.epsilon){
								edgeScore = edgeScore + parameter.edgeMismatch;							
							}
						}
						if( (alignment[k][i] == -1 || alignment[k][j] == -1) && alignment[l][i] != -1 && alignment[l][j] != -1){
							edgeScore = edgeScore + parameter.edgeMismatch;
						}
						if( (alignment[l][i] == -1 || alignment[l][j] == -1) && alignment[k][i] != -1 && alignment[k][j] != -1){
							edgeScore = edgeScore + parameter.edgeMismatch;
						}
					}
				}
			}
		}
		return edgeScore;
	}
	
	
	/**Calculates node score
	 * 
	 */
	private static double nodeScore(int[][] alignment, Graph[] graphs, EvalParameter parameter){
		double nodeScore = 0;
		int columns = alignment[0].length;
		int rows = alignment.length;
		for(int i = 0; i < columns; i++){
			for(int j = 0; j < rows; j++){
				for(int k = j+1; k < rows; k++){
					if(alignment[j][i] != -1 && alignment[k][i] != -1 && graphs[j].nodes[alignment[j][i]].compareTo(graphs[k].nodes[alignment[k][i]]) == 0){
						nodeScore = nodeScore + parameter.match;
					}
					if(alignment[j][i] != -1 && alignment[k][i] != -1 && graphs[j].nodes[alignment[j][i]].compareTo(graphs[k].nodes[alignment[k][i]]) != 0){
						nodeScore = nodeScore + parameter.mismatch;
					}
					if( (alignment[j][i] == -1 && alignment[k][i] != -1) || (alignment[j][i] != -1 && alignment[k][i] == -1) ){
						nodeScore = nodeScore + parameter.dummy;
					}
				}
			}
		}
		return nodeScore;
	}
}