/*
 * Created on 06.06.2008
 */
package Utilities;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.OutputStreamWriter;
import java.util.Arrays;
import java.util.Vector;

import Parser.ParseGraphs;
import Utilities.Graph;

/**
 * @author Thomas Fober
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         thomas@mathemetik.uni-marburg.de
 */
public class CreateDistMatrix {
	
	double[][] distances;
	Vector<Integer> pos;
	Vector<Integer> neg;
	String[] dataN;
	int[] numberOfNodes;
	
	public static void main(String[] args) {
		String results = args[0];
		String positives = args[1];
		String negatives = args[2];
		CreateDistMatrix dist = new CreateDistMatrix(results, positives, negatives);
		System.out.println("");
	}
	
	public double valueOf(String path){
		String text = null;
		try{
			BufferedReader file = new BufferedReader(new FileReader(path));
			for(int i = 0; i < 5; i++){
				text = file.readLine();
			}
			return Double.parseDouble((text.split(":"))[1]);
		}catch(Exception e){
			System.out.println("Error reading results");
		}
		return 0;
	}
	
	public boolean contain(String[] data, String name){
		for(int i = 0; i < data.length; i++){
			if(data[i].equals(name)) return true;
		}
		return false;
	}
	
	public CreateDistMatrix(String results, String positives, String negatives){
		//distance matrix
		File path = new File(results);
		String[] data = path.list();
		Arrays.sort(data);
		int dataLength = (int)(-0.5 + Math.sqrt(0.25 + 2 * data.length));
		int k = 0;
		distances = new double[dataLength][dataLength];
		for(int i = 0; i < dataLength; i++){
			for(int j = i; j < dataLength; j++){
				distances[i][j] = valueOf(results+"\\"+data[k]);
				k++;
			}
		}
		for(int i = 0; i < distances.length; i++){
			for(int j = 0; j < distances.length; j++){
				distances[j][i] = distances[i][j];
			}
		}
		
		//pos-/neg-vector
		String[] names = new String[dataLength];
		String name;
		File pathPos = new File(positives);
		String[] dataPos = pathPos.list();
		Arrays.sort(dataPos);
		File pathNeg = new File(negatives);
		String[] dataNeg = pathNeg.list();
		Arrays.sort(dataNeg);
		pos = new Vector<Integer>();
		neg = new Vector<Integer>();
		dataN = new String[dataLength];
		for(int i = 0; i < dataLength; i++){
			name = data[i].split("___")[1];
			name = name.substring(0, name.lastIndexOf(".rw"));
			this.dataN[i] = name;
			if(contain(dataPos, name)) pos.add(i); 
			if(contain(dataNeg, name)) neg.add(i);
		}
		
		//////////////////////
		//newNormalizaion();//
		//////////////////////
		
		//write
		try{
	    	BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream("C:\\Documents and Settings\\thomas\\Desktop\\dist.txt")));
	    	for(int i = 0; i < distances.length; i++){
	    		for(int j = 0; j < distances[0].length; j++){
	    			if(j != distances[0].length - 1) out.write(((Double)distances[i][j]).toString() + "\t");
	    			else out.write(((Double)distances[i][j]).toString());
	    		}
	    		out.newLine();
	    	}
	    	out.close();
	    }catch(Exception e){
	    	System.out.println("Error writing distances");
	    }
	    try{
	    	BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream("C:\\Documents and Settings\\thomas\\Desktop\\pos.txt")));
	    	for(int i = 0; i < pos.size(); i++){
	    		if(i != pos.size() - 1) out.write(((Integer)pos.get(i)).toString() + "\t");
	    		else out.write(((Integer)pos.get(i)).toString());
	    	}
	    	out.close();
	    }catch(Exception e){
	    	System.out.println("Error writing positives");
	    }
	    try{
	    	BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream("C:\\Documents and Settings\\thomas\\Desktop\\neg.txt")));
	    	for(int i = 0; i < neg.size(); i++){
	    		if(i != neg.size() - 1) out.write(((Integer)neg.get(i)).toString() + "\t");
	    		else out.write(((Integer)neg.get(i)).toString());
	    	}
	    	out.close();
	    }catch(Exception e){
	    	System.out.println("Error writing negatives");
	    }
	    ///////////////////////////////////////
	    leaveOneOutTest(distances, pos, neg);//
	    ///////////////////////////////////////
	}
	
	public void newNormalizaion(){
		numberOfNodes = new int[dataN.length];
		Graph g;
		for(int i = 0; i < dataN.length; i++){
			g = ParseGraphs.parseFile("C:\\Documents and Settings\\thomas\\Desktop\\Subset" + "\\" + dataN[i], 11);
			numberOfNodes[i] = g.numberOfNodes;
		}
		for(int i = 0; i < distances.length; i++){
			for(int j = 0; j < distances[0].length; j++){
				//distances[i][j] = distances[i][j] / min(numberOfNodes[i], numberOfNodes[j]);
				distances[i][j] = distances[i][j] / -min(-numberOfNodes[i], -numberOfNodes[j]);
				//distances[i][j] = distances[i][j] / (numberOfNodes[i] + numberOfNodes[j]);
			}
		}
	}
	
	public int min(int a, int b){
		if(a > b) return b;
		else return a;
	}
	
	public int knn(Vector<Integer> positive, Vector<Integer> negative, int querry, int k){
		double[] distance = new double[distances.length];
		int sum = 0;
		int result;
		for(int i = 0; i < distances.length; i++){
			if(querry == i){
				distance[i] = Double.NEGATIVE_INFINITY;
			}else{
				distance[i] = distances[i][querry];
			}
		}
		
		for(int i = 0; i < k; i++){
			result = searchBest(distance);
			sum = sum + (positive.contains(result) ? 1 : -1);
			distance[result] = Double.NEGATIVE_INFINITY;
		}
		if(sum < 0) return -1;
		else if(sum > 0) return 1;
		else if(positive.size() > negative.size()) return 1;
		else return -1;
	}
	
	public static int searchBest(double[] values){
		double bestValue = Double.NEGATIVE_INFINITY;
		int best = 0;
		for(int i = 0; i < values.length; i++){
			if(values[i] > bestValue){
				bestValue = values[i];
				best = i;
			}
		}
		return best;
	}
	
	public void leaveOneOutTest(double[][] distances, Vector<Integer> pos, Vector<Integer> neg){
		int result;
		double correct = 0;
		for(int leave = 0; leave < distances.length; leave++){
			 result = knn(pos, neg, leave, 1);
			 if(result == 1 && pos.contains(leave)) correct++;
			 if(result == -1 && neg.contains(leave)) correct++;
		}
		System.out.println(correct / distances.length);
	}
	
}
