/*
 * Created on 10.08.2007
 */
package Utilities;
import java.io.*;
import java.util.ArrayList;
import java.util.Vector;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;

/**
 * @author Thomas Fober & Marco Mernberger
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         {thomas, mernberger}@mathemetik.uni-marburg.de
 */
public class Solution implements Serializable{
	
	public static final long serialVersionUID = -8218943250087807302L;
	
	//Matrix of node indices
	public int[][] solution;
	
	//fitness score of solution
	public double value;
	
	//elapsed calculation time
	public long time;
	
	//number of generations, if calculated by EA method
	public int generations;
	
	//list of input graphs
	public Graph[] graphs;
	
	public String method;
	
	public String name;
	
	public String reason;
	
	public EvalParameter parameter;
	

	/**Default constructor
	 */
	public Solution(){
	}
	
	
	/**Constructs a solution from a Vector list, containing all rows of solution.
	 * @param alig list of rows
	 * @param graphs list of input graphs
	 * @param time elapsed calculation time
	 */
	public Solution(Vector<Integer>[] alig, double score, Graph[] graphs, long time, String method, String reason, EvalParameter parameter) {
		this.method = method;
		this.reason = reason;
		this.parameter = parameter;
		int[][] sol = Solution.vectorToSol(alig);
		this.time = time;
		this.solution = sol;
		this.graphs = graphs;
		if (graphs.length <= 2) this.name = graphs[0].name + "___" + graphs[1].name + "_"+method;
		else this.name = graphs[0].name+"...multiple";
		this.value = Math.round(100 * score);
		this.value /= 100;
	}
	
	/**Constructs a Solution1 
	 * 
	 * @param pathToMGAFile path to .mga file
	 */
	public Solution (String pathToMGAFile) {
		super();
		try {
			BufferedReader in = new BufferedReader(new FileReader(pathToMGAFile));
			String next = in.readLine();
			next = in.readLine();
			String num = next.substring(21);
			next = in.readLine();
			String method = next.substring(21);
			next = in.readLine();
			String time = next.substring(21);
			next = in.readLine();
			String value = next.substring(21);
			next = in.readLine();
			String generations = next.substring(21);
			next = in.readLine();
			String reason = next.substring(21);
			next = in.readLine();
			String tmp = next.substring(43);
			String[] para = tmp.split(", ");
			EvalParameter parameter = new EvalParameter(para);
			in.readLine();
			//reading alignment matrix
			Vector<Integer[]> vec = new Vector<Integer[]>(); 
			num = num.trim();
			for (int i = 0; i < Integer.parseInt(num); i++) {
				next = in.readLine();
				String[] linesplit = next.split(";");
				Integer[] line = new Integer[linesplit.length];
				for (int j = 0; j < linesplit.length; j++) {
					line[j] = Integer.parseInt(linesplit[j]);
				}
				vec.add(line);
			}
			int[][] sol = Solution.vectorToSol1(vec);
			// reading graphs
			Vector<Graph> graphs = new Vector<Graph>();
			for (int i = 0; i < Integer.parseInt(num); i++) {
				in.readLine();
				next = in.readLine();
				next = in.readLine();
				String[] nodes = next.split(";");
				//reading distance matrix
				next = in.readLine();
				Vector<Double[]> dist = new Vector<Double[]>();
				next = in.readLine();
				String[] linesplit = next.split(";");
				int numOfNodes = linesplit.length;
				
				for (int j = 0; j < numOfNodes; j++) {
					Double[] line = new Double[numOfNodes];
					for (int k = 0; k < linesplit.length; k++) {
						line[k] = Double.parseDouble(linesplit[k]);
					}
					dist.add(line);
					if(j == numOfNodes-1){
						break;
					}
					next = in.readLine();
					linesplit = next.split(";");	
				}
				Graph g = new Graph(numOfNodes, null, Solution.vectorToSol(dist), nodes, null, pathToMGAFile);
				graphs.add(g);
				//next = in.readLine();
			}
			this.generations = Integer.parseInt(generations.trim());
			this.value = Double.parseDouble(value.trim());
			this.graphs = new Graph[graphs.size()];
			for(int i = 0; i < graphs.size(); i++){
				this.graphs[i] = graphs.get(i);
			}
			this.solution = sol;
			this.time = Long.parseLong(time.trim());
			this.method = method;
			this.reason = reason;
			this.parameter = parameter;
			String[] split;
			if(File.separator.equals("\\")){
				split = pathToMGAFile.split(File.separator+File.separator);
			}else{
				split = pathToMGAFile.split(File.separator);
			}			
			this.name = split[split.length-1].substring(0, split[split.length-1].length()-5);
		}
		catch (IOException e) {
			System.out.println(e);
		}
	}	
	
	/**converts solution to array of lists
	 * 
	 * @return alignment as Array of dynamic length lists
	 */
	public static  ArrayList<Integer>[] solToArray(int[][] sol){
		ArrayList<Integer>[] ret = new ArrayList[sol.length];
		for (int i = 0; i < sol.length; i++){
			ArrayList<Integer> next = new ArrayList<Integer>();
			for (int j = 0; j < sol[i].length; j++){
				next.add(sol[i][j]);
			}
			ret[i] = next;
		}
		return ret;
	}

	
	/**Converts a list of Vectors to int[][] 
	 * 
	 * @param vec list of vectors
	 * @return alignment matrix
	 */
	public static int[][] vectorToSol1(Vector<Integer[]> vec) {
		int[][] sol = new int[vec.size()][vec.firstElement().length];
		for (int i = 0; i < vec.size(); i++) {
			for (int j = 0; j < vec.firstElement().length; j++) {
				sol[i][j] = vec.get(i)[j]; 
			}
		}
		return sol;
	}
	
	/**Converts a list of Vectors to double[][] 
	 * 
	 * @param vec list of vectors
	 * @return alignment matrix
	 */
	public static double[][] vectorToSol(Vector<Double[]> vec) {
		double[][] sol = new double[vec.size()][vec.firstElement().length];
		for (int i = 0; i < vec.size(); i++) {
			for (int j = 0; j < vec.firstElement().length; j++) {
				sol[i][j] = vec.get(i)[j]; 
			}
		}
		return sol;
	}
	
	/**Converts a list of Vectors to int[][] 
	 * 
	 * @param vec list of vectors
	 * @return alignment matrix
	 */
	public static int[][] vectorToSol(Vector<Integer>[] vec) {
		int maxlength = 0;
		for (Vector<Integer> test : vec) {
			if(test.size() > maxlength) maxlength = test.size();
		}
		int[][] sol = new int[vec.length][maxlength];
		for (int i = 0; i < vec.length; i++) {
			for (int j = 0; j < vec[0].size(); j++) {
				sol[i][j] = vec[i].get(j); 
			}
		}
		return sol;
	}
	/**Simple to String method for displaying solutions in text format.
	 * 
	 */
	public String toString(){
		String indi = "";
		indi +="#MGA "+name+"\n";
		indi +="#Number of graphs  	: "+this.solution.length+"\n";
		indi +="#Calculation method	: "+this.method+"\n";
		indi +="#Calculation time  	: "+this.time+"\n";
		indi +="#Score             	: "+this.value+"\n";
		indi +="#Generations       	: "+this.generations+"\n";
		indi +="stop reason        	: "+this.reason+"\n";
		indi +="Scoring parameter		: "+this.parameter+"\n";
		indi += indi + "Alignment(indices):\n";
		for (int i = 0; i < solution.length; i++){	
			for (int s : solution[i]){
				indi = indi + (String.valueOf(s)+" | ");
			}
			indi = indi + ("\n");
		}
		indi = indi + "Alignment(nodes):\n";
		for (int i = 0; i < solution.length; i++){	
			for (int s : solution[i]){
				if (s == -1) indi = (indi + "-1 | ");
				else indi = indi + (graphs[i].nodes[s]+" | ");
			}
			indi = indi + ("\n");
		}
		return indi;
	}
	
	/**Saves the Solution1 with default name for display in graph drawing tool, using a csv standard.
	 * 
	 * @param path directory path for file output
	 */
	public void save(String path){
		this.save(path, this.name);
	}
	
	/**Saves the Solution1 for display in graph drawing tool, using a csv standard.
	 * 
	 * @param path directory path for file output
	 * @param name name of file
	 */
	public void save(String path, String name){
		try {
			BufferedWriter alig = new BufferedWriter(new FileWriter(path+File.separator+name+".mga"));
			alig.write("#MGA "+name+"\n");
			alig.write("#Number of graphs  	: "+this.solution.length+"\n");
			alig.write("#Calculation method	: "+this.method+"\n");
			alig.write("#Calculation time  	: "+this.time+"\n");
			alig.write("#Score             	: "+this.value+"\n");
			alig.write("#Generations       	: "+this.generations+"\n");
			alig.write("stop reason        	: "+this.reason+"\n");
			alig.write("Scoring parameter		: "+this.parameter+"\n");
			alig.write("#Alignment matrix\n");
			for (int i = 0; i < this.solution.length; i++) {
				String line = "";
				for (int j = 0; j < this.solution[0].length-1; j++) {
					line += solution[i][j] +";";
				}
				line += solution[i][solution[0].length-1];
				alig.write(line+"\n");
			}
			for (Graph g : this.graphs) {
				alig.write("#graph "+g.name+"\n");
				alig.write("#labels\n");
				String line = "";
				for (int i = 0; i < g.nodes.length-1; i++) {
						line += g.nodes[i] +";";
					}
				line += g.nodes[g.nodes.length-1];
				alig.write(line+"\n");
				alig.write("#distance\n");
				for (int i = 0; i < g.distances.length; i++) {
					line = "";
					for (int j = 0; j < g.distances[0].length-1; j++) {
						line += g.distances[i][j] +";";
					}
					line += g.distances[i][g.distances[0].length-1];
					alig.write(line+"\n");
				}
			}
			alig.close();
		}
		catch(IOException e) {
			System.out.println(e);
		}
	}
	
	
	public String mga() {
		String alig = "";
		alig += "#MGA "+name+"\n";
		alig += "#Number of graphs  	: "+this.solution.length+"\n";
		alig += "#Calculation method	: "+this.method+"\n";
		alig += "#Calculation time  	: "+this.time+"\n";
		alig += "#Score             	: "+this.value+"\n";
		alig += "#Generations       	: "+this.generations+"\n";
		alig += "stop reason        	: "+this.reason+"\n";
		alig += "Scoring parameter		: "+this.parameter+"\n";
		alig += "#Alignment matrix\n";
		for (int i = 0; i < this.solution.length; i++) {
			String line = "";
			for (int j = 0; j < this.solution[0].length-1; j++) {
				line += solution[i][j] +";";
			}
			line += solution[i][solution[0].length-1];
			alig += line+"\n";
		}
		for (Graph g : this.graphs) {
			alig += "#graph "+g.name+"\n";
			alig += "#labels\n";
			String line = "";
			for (int i = 0; i < g.nodes.length-1; i++) {
					line += g.nodes[i] +";";
				}
			line += g.nodes[g.nodes.length-1];
			alig += line+"\n";
			alig += "#distance\n";
			for (int i = 0; i < g.distances.length; i++) {
				line = "";
				for (int j = 0; j < g.distances[0].length-1; j++) {
					line += g.distances[i][j] +";";
				}
				line += g.distances[i][g.distances[0].length-1];
				alig += line+"\n";
			}
		}
		return alig;
	}
	
	
	public void saveSerial(String path) {
		this.saveSerial(path, this.name);
	}
	
	public void saveSerial(String path, String filename) {
		try{
			String file = filename+".ser";
			FileOutputStream f = new FileOutputStream(path + File.separator + file);
			ObjectOutputStream out = new ObjectOutputStream(f);
			out.writeObject(this);
			out.close();
		}
		catch(IOException e){
			System.out.println(e);
		}
	}
}



